@@ -1,18 +1,13 @@
-#coding: utf-8
+# coding: utf-8
+from __future__ import absolute_import
 
 from simple_report.converter.abstract import FileConverter
 from simple_report.converter.open_office.wrapper import OOWrapper
 
-__author__ = 'prefer'
 
 class OpenOfficeConverter(FileConverter):
-    """
-
-    """
 
     def __init__(self, port):
-        """
-        """
         super(OpenOfficeConverter, self).__init__()
         self.port = port
 
@@ -26,4 +21,3 @@ class OpenOfficeConverter(FileConverter):
         #        if to_format in (FileConverter.XLSX, FileConverter.DOCX):
         #            raise FileConverterException('Format "%s" not supported' % to_format)
         return OOWrapper(self.port).convert(self.file.get_path(), to_format)
-
