@@ -1,8 +1,2 @@
-#coding: utf-8
-"""
-Created on 17.12.2011
-
-@author: prefer
-"""
-
-XML_DEFINITION = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
\ No newline at end of file
+# coding: utf-8
+XML_DEFINITION = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
