@@ -1,16 +1,20 @@
-#coding: utf-8
+# coding: utf-8
+from __future__ import absolute_import
+
+from abc import ABCMeta
+from abc import abstractmethod
+
+import six
 
-from abc import ABCMeta, abstractmethod
 from simple_report.interface import ISpreadsheetSection
 from simple_report.utils import ColumnHelper
 
-class AbstractCursor(object):
+
+class AbstractCursor(six.with_metaclass(ABCMeta, object)):
     """
     Абстрактный курсор для табличных отчетов.
     """
 
-    __metaclass__ = ABCMeta
-
     @abstractmethod
     def __init__(self):
         """
@@ -52,13 +56,11 @@ class AbstractCursor(object):
 
         assert isinstance(value[1], int) # Проверка координаты строки.
 
-class AbstractCalculateNextCursor(object):
+class AbstractCalculateNextCursor(six.with_metaclass(ABCMeta, object)):
     """
     Класс, который занимается работой по вычислению курсоров
     """
 
-    __metaclass__ = ABCMeta
-
 
     def get_next_cursor(self, cursor, begin, end, oriented, section=None):
         """
