@@ -1,14 +1,16 @@
-#coding: utf-8
+# coding: utf-8
+from __future__ import absolute_import
 
 import abc
 
-class AbstractSectionException(Exception):
+import six
+
+
+class AbstractSectionException(six.with_metaclass(abc.ABCMeta, Exception)):
     """
     Абстрактный класс для исключений, которые возникают при работе с секциями
     """
 
-    __metaclass__ = abc.ABCMeta
-
 class SectionException(AbstractSectionException):
     """
     Исключение работы с секциями
@@ -21,13 +23,11 @@ class SectionNotFoundException(AbstractSectionException):
     """
 
 
-class SheetException(Exception):
+class SheetException(six.with_metaclass(abc.ABCMeta, Exception)):
     """
     Абстрактный класс для исключений, которые возникают при работе с листами таблицы
     """
 
-    __metaclass__ = abc.ABCMeta
-
 
 class SheetNotFoundException(SheetException):
     """
