@@ -1,13 +1,15 @@
-#coding: utf-8
+# coding: utf-8
+from __future__ import absolute_import
 
 import abc
 
-from simple_report.utils import ColumnHelper
+import six
+
 from simple_report.interface import ISpreadsheetSection
+from simple_report.utils import ColumnHelper
 
-class SpreadsheetSection(object):
 
-    __metaclass__ = abc.ABCMeta
+class SpreadsheetSection(six.with_metaclass(abc.ABCMeta, object)):
 
     def __init__(self, sheet, name, begin=None, end=None):
         """
@@ -40,13 +42,11 @@ class SpreadsheetSection(object):
         return indent
 
 
-class AbstractMerge(object):
+class AbstractMerge(six.with_metaclass(abc.ABCMeta, object)):
     """
     Конструкция Merge
     """
 
-    __metaclass__ = abc.ABCMeta
-
     def __init__(self, parent_section, child_section, params, oriented=ISpreadsheetSection.LEFT_DOWN):
 
         self.section = parent_section
