@@ -1,8 +1,8 @@
 # coding: utf-8
+from __future__ import absolute_import
 
 from datetime import datetime
 
-__author__ = 'prefer'
 
 class TemplateTags(object):
     """
