@@ -1,17 +1,18 @@
-#coding: utf-8
+# coding: utf-8
+from __future__ import absolute_import
 
+from abc import ABCMeta
+from abc import abstractmethod
 import os
-from abc import ABCMeta, abstractmethod
+
 from lxml.etree import parse
-
-__author__ = 'prefer'
+import six
 
 
-class OpenXMLFile(object):
+class OpenXMLFile(six.with_metaclass(ABCMeta, object)):
     """
     Файл XML в структуре OpenXML
     """
-    __metaclass__ = ABCMeta
 
     NS = None
 
@@ -64,11 +65,10 @@ class OpenXMLFile(object):
         return cls(*args, **kwargs)
 
 
-class ReletionOpenXMLFile(OpenXMLFile):
+class ReletionOpenXMLFile(six.with_metaclass(ABCMeta, OpenXMLFile)):
     """
     Файл связей
     """
-    __metaclass__ = ABCMeta
 
     RELETION_EXT = '.rels'
     RELETION_FOLDER = '_rels'
@@ -85,15 +85,13 @@ class ReletionOpenXMLFile(OpenXMLFile):
             self._reletion_root = self.from_file(rel_path)
 
 
-class CommonProperties(ReletionOpenXMLFile):
+class CommonProperties(six.with_metaclass(ABCMeta, ReletionOpenXMLFile)):
     u"""
     Общие настройки
 
     Находит папку _rels и парсит файл .rels в нем
     """
 
-    __metaclass__ = ABCMeta
-
     #
     NS = "http://schemas.openxmlformats.org/package/2006/relationships"
 
