@@ -1,16 +1,20 @@
-#!coding:utf-8
+# coding: utf-8
+from __future__ import absolute_import
 
 import copy
 import os
-from lxml.etree import tostring, SubElement
-from simple_report.core import XML_DEFINITION
-from simple_report.core.exception import (
-    SectionNotFoundException, SectionException)
-from simple_report.core.xml_wrap import (
-    ReletionOpenXMLFile, CommonProperties, OpenXMLFile)
-from simple_report.docx.drawing import DocxImage, insert_image
 
-__author__ = 'prefer'
+from lxml.etree import SubElement
+from lxml.etree import tostring
+import six
+
+from simple_report.core.exception import SectionException
+from simple_report.core.exception import SectionNotFoundException
+from simple_report.core.xml_wrap import CommonProperties
+from simple_report.core.xml_wrap import OpenXMLFile
+from simple_report.core.xml_wrap import ReletionOpenXMLFile
+from simple_report.docx.drawing import DocxImage
+from simple_report.docx.drawing import insert_image
 
 
 class Wordprocessing(ReletionOpenXMLFile):
@@ -36,8 +40,10 @@ class Wordprocessing(ReletionOpenXMLFile):
         """
         Сборка файла
         """
-        with open(self.file_path, 'w') as f:
-            f.write(XML_DEFINITION + tostring(self._root))
+        with open(self.file_path, 'wb') as f:
+            f.write(
+                tostring(self._root, encoding='utf-8', xml_declaration=True)
+            )
 
     def set_params(self, params):
         """
@@ -93,24 +99,43 @@ class Wordprocessing(ReletionOpenXMLFile):
                         if old_node is not None and old_signature == signature:
                             # delete r node
                             old_node[1].text = old_node[1].text + node.text
-                            #old_node = (par_node, node)
                             paragraph.remove(par_node)
                         else:
                             old_node = (par_node, node)
 
+    @classmethod
+    def split_into_paragraphs(cls, text_node, text_rows):
+        """
+            Разбивает текущий текст на несколько параграфов
+            (для отображения текста в несколько строчек)
+        """
+        text_node.text = text_rows[0]
+        paragraph = list(
+            text_node.iterancestors(
+                '{' + Wordprocessing.NS_W + '}p'))[0]
+
+        for text in text_rows[1:]:
+            p_copy = copy.copy(paragraph)
+            paragraph.addnext(p_copy)
+            t_node = p_copy.find(
+                './/{0}:r/{0}:t'.format('w'),
+                namespaces={'w': Wordprocessing.NS_W}
+            )
+            t_node.text = text
+            paragraph = p_copy
+            
     @classmethod
     def _set_params(cls, text_nodes, params, doc_rels=None):
 
-        def sorting_key((k, v)):
-            if not isinstance(k, basestring):
+        def sorting_key(item):
+            key, _ = item
+            if not isinstance(key, six.string_types):
                 return 1
-            return -len(k)
+            return -len(key)
 
         for node in text_nodes:
-            for key_param, value in sorted(params.items(), key=sorting_key):
+            for key_param, value in sorted(six.iteritems(params), key=sorting_key):
                 if key_param in node.text:
-                    # if len(node.text) > 0 and node.text[0] == '#' and
-                    # node.text[-1] == '#':
                     text_to_replace = '#%s#' % key_param
                     if text_to_replace in node.text:
                         if isinstance(value, DocxImage):
@@ -118,11 +143,16 @@ class Wordprocessing(ReletionOpenXMLFile):
                                 node, value, text_to_replace, doc_rels)
                         else:
                             node.text = node.text.replace(
-                                text_to_replace, unicode(value))
+                                text_to_replace, six.text_type(value))
                     else:
                         node.text = node.text.replace(
-                            key_param, unicode(value))
-
+                            key_param, six.text_type(value))
+                    
+                    # Проверим наличие символов новой строки в тексте
+                    text_rows = node.text.split('\n')
+                    if len(text_rows) > 1:
+                        cls.split_into_paragraphs(node, text_rows)
+                        
     def get_all_parameters(self):
         """
         Получение всех параметров
@@ -168,9 +198,9 @@ class Wordprocessing(ReletionOpenXMLFile):
                 )
                 if not col_nodes:
                     continue
-                col_node = col_nodes[0]
-                if col_node.text and col_node.text[:2] == '#!':
-                    section_name = col_node.text[2:]
+                col_nodes_text = u''.join(x.text for x in col_nodes)
+                if col_nodes_text and col_nodes_text[:2] == '#!':
+                    section_name = col_nodes_text[2:]
                     if section_name in self.table_sections:
                         raise SectionException(
                             ('Section named {0} has been found '
@@ -179,15 +209,13 @@ class Wordprocessing(ReletionOpenXMLFile):
                             )
                         )
                     rows_to_delete.append(row_node)
-                    # del text_nodes[text_nodes.index(row_node)]
-                elif col_node.text and col_node.text[-2:] == '!#':
+                elif col_nodes_text and col_nodes_text[-2:] == '!#':
                     self.table_sections[section_name] = section
                     section_name = None
                     section = Section(table)
                     rows_to_delete.append(row_node)
                 elif section_name:
                     section.append(copy.copy(row_node))
-                    # del text_nodes[text_nodes.index(row_node)]
                     rows_to_delete.append(row_node)
             for row_node in rows_to_delete:
                 row_node.getparent().remove(row_node)
@@ -227,7 +255,6 @@ class DocumentRelsXMLFile(OpenXMLFile):
         if os.path.exists(file_path):
             self._root = self.from_file(file_path)
             for child in self._root:
-                # print child.tag, dict(child.attrib), 'child'
                 attrib = dict(child.attrib)
                 rid = attrib.get('Id', '')
                 if rid.startswith('rId'):
@@ -254,8 +281,10 @@ class DocumentRelsXMLFile(OpenXMLFile):
         return cls(tags, rel_id, folder, file_name, reletion_path, )
 
     def build(self):
-        with open(self.file_path, 'w') as file_:
-            file_.write(XML_DEFINITION + tostring(self._root))
+        with open(self.file_path, 'wb') as file_:
+            file_.write(
+                tostring(self._root, encoding='utf-8', xml_declaration=True)
+            )
 
 
 
@@ -305,9 +334,6 @@ class ContentTypesXMLFile(OpenXMLFile):
                     'ContentType': 'image/jpeg',
                     'Extension': 'jpg'
                 },
-                # nsmap={
-                #     'w': self.NS
-                # }
             )
         if not found_png:
             SubElement(
@@ -315,13 +341,12 @@ class ContentTypesXMLFile(OpenXMLFile):
                     'ContentType': 'image/png',
                     'Extension': 'png'
                 },
-                # nsmap={
-                #     'w': self.NS
-                # }
             )
 
-        with open(self.file_path, 'w') as f:
-            f.write(XML_DEFINITION + tostring(root))
+        with open(self.file_path, 'wb') as f:
+            f.write(
+                tostring(root, encoding='utf-8', xml_declaration=True)
+            )
 
 
     @classmethod
