@@ -1,24 +1,23 @@
 # coding: utf-8
-"""
-Created on 24.11.2011
-
-@author: prefer
-"""
+from __future__ import absolute_import
 
 import abc
 import os
+
+import six
+
+from simple_report.converter.abstract import FileConverter
 from simple_report.core.document_wrap import BaseDocument
 from simple_report.core.exception import WrongDocumentType
 from simple_report.core.tags import TemplateTags
 from simple_report.docx.document import DocumentDOCX
-
-from simple_report.interface import ISpreadsheetReport, IDocumentReport
-from simple_report.converter.abstract import FileConverter
+from simple_report.interface import IDocumentReport
+from simple_report.interface import ISpreadsheetReport
 from simple_report.rtf.document import DocumentRTF
-from simple_report.xlsx.document import DocumentXLSX
+from simple_report.utils import FileProxy, ColumnHelper
 from simple_report.xls.document import DocumentXLS
 from simple_report.xls.output_options import XSL_OUTPUT_SETTINGS
-from simple_report.utils import FileProxy
+from simple_report.xlsx.document import DocumentXLSX
 
 
 class ReportGeneratorException(Exception):
@@ -27,13 +26,11 @@ class ReportGeneratorException(Exception):
     """
 
 
-class Report(object):
+class Report(six.with_metaclass(abc.ABCMeta, object)):
     u"""
     Абстрактный класс отчета
     """
 
-    __metaclass__ = abc.ABCMeta
-
     def __init__(self, src_file, converter=None, tags=None, wrapper=None,
                  type=None, **kwargs):
         """
@@ -210,6 +207,20 @@ class SpreadsheetReport(Report, ISpreadsheetReport):
             raise WrongDocumentType()
         return self._wrapper.get_section(section_name)
 
+    def build(self, *args, **kwargs):
+        u"""
+        Дополнительные действия перед построением отчета.
+        """
+        self._reset_caches()
+
+        return super(SpreadsheetReport, self).build(*args, **kwargs)
+
+    def _reset_caches(self):
+        u"""
+        Сброс кэшей
+        """
+        ColumnHelper.reset_caches()
+
     @property
     def workbook(self):
         u"""
