@@ -1,9 +1,10 @@
-#!coding:utf-8
-from simple_report.core.document_wrap import BaseDocument
-from simple_report.rtf.parser import normalize_rtf
-from simple_report.rtf.api import convert_dict, do_replace
+# coding: utf-8
+from __future__ import absolute_import
 
-__author__ = 'khalikov'
+from simple_report.core.document_wrap import BaseDocument
+from simple_report.rtf.api import convert_dict
+from simple_report.rtf.api import do_replace
+from simple_report.rtf.parser import normalize_rtf
 
 
 class DocumentRTF(BaseDocument):
