@@ -1,10 +1,11 @@
-#coding: utf-8
+# coding: utf-8
+from __future__ import absolute_import
 
-import xlrd
-
-from simple_report.core.document_wrap import BaseDocument, SpreadsheetDocument
-from simple_report.xls.workbook import Workbook
+from simple_report.core.document_wrap import BaseDocument
+from simple_report.core.document_wrap import SpreadsheetDocument
 from simple_report.xls.output_options import XSL_OUTPUT_SETTINGS
+from simple_report.xls.workbook import Workbook
+
 
 class DocumentXLS(BaseDocument, SpreadsheetDocument):
     """
