@@ -1,22 +1,27 @@
-#coding:utf-8
+# coding: utf-8
+from __future__ import absolute_import
 
-from datetime import datetime, date, time
+from datetime import date
+from datetime import datetime
+from datetime import time
 import decimal
-
 import re
-import xlrd
-import xlwt
-from xlwt.Style import default_style
-from simple_report.interface import ISpreadsheetSection
-
-from simple_report.core.exception import XLSReportWriteException
-from simple_report.core.spreadsheet_section import (
-    SpreadsheetSection, AbstractMerge
-)
-from simple_report.xls.cursor import CalculateNextCursorXLS
-from simple_report.utils import FormulaWriteExcel
 
 from xlwt import Formatting as xlwt_formatting
+from xlwt.Style import default_style
+import m3_xlrd
+import xlwt
+
+from six.moves import range
+import six
+
+from simple_report.core.exception import XLSReportWriteException
+from simple_report.core.spreadsheet_section import AbstractMerge
+from simple_report.core.spreadsheet_section import SpreadsheetSection
+from simple_report.interface import ISpreadsheetSection
+from simple_report.utils import FormulaWriteExcel
+from simple_report.xls.cursor import CalculateNextCursorXLS
+
 
 # for k, v in xlwt.ExcelMagic.all_funcs_by_name.items():
 #     xlwt.ExcelMagic.all_funcs_by_name[k] = list(v)
@@ -55,13 +60,16 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
         self.writer = writer
 
     def flush(self, params, oriented=ISpreadsheetSection.LEFT_DOWN,
-              used_formulas=None):
+              used_formulas=None, keep_text_type=KEEP_TEXT_TYPE,
+              clean_template_tags=True):
         """
          Запись секции в отчет
         :param params: словарь с параметрами подстановки
         :param oriented: направление вывода секции
         :param used_formulas: используемые формулы - нужны для записи
         простых формул в отчет
+        :param clean_template_tags: очищать ли теги для которых не были
+        переданы значения
         :result: None
         """
         for k, v in params.items():
@@ -99,7 +107,7 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
                 cty = cell.ctype
                 f_id = None
                 for key, value in params.items():
-                    if unicode(cell.value).count(u''.join(['#', key, '#'])):
+                    if six.text_type(cell.value).count(u''.join(['#', key, '#'])):
                         if used_formulas:
                             formula_id_list = used_formulas.get(key)
                             if formula_id_list:
@@ -107,7 +115,7 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
                                     self.sheet_data.formula_id_dict.setdefault(
                                         formula_id, []
                                     ).append(
-                                        ''.join([xlrd.colname(wtcolx),
+                                        ''.join([m3_xlrd.colname(wtcolx),
                                                 str(wtrowx + 1)])
                                     )
 
@@ -133,7 +141,7 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
                                     cty = FORMULA_XLS_TYPE
                                 else:
                                     val = ''
-                                    cty = xlrd.XL_CELL_TEXT
+                                    cty = m3_xlrd.XL_CELL_TEXT
                                 break
                         elif isinstance(value, XLSImage):
                             cty = EXCEL_IMAGE_TYPE
@@ -142,12 +150,19 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
                         # Тип ячейки
                         cty = self.get_value_type(value=value,
                                                   default_type=cell.ctype)
-                        value = unicode(value)
+                        value = six.text_type(value)
                         val = val.replace(u'#%s#' % key, value)
 
-                if isinstance(val, basestring):
+                # Убирает тэги для которых не были переданы значения
+                # Нужно отключать этот функционал
+                # (передавать clean_template_tags=False)
+                # если в отчёт может быть записан текст подходящий
+                # под регулярку /#.*#/ чтобы избежать его удаления
+                # в этом случае для удаления ненужных тэгов достаточно
+                # просто передать пустую строку
+                if isinstance(val, six.string_types) and clean_template_tags:
                     while u'#' in val:
-                        val = re.sub(u'#.*#', '', val)
+                        val = re.sub(u'#.*#', '', val, flags=re.DOTALL)
 
                         if len(val.split('#')) == 2:
                             break
@@ -167,7 +182,7 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
 
                     self.writer.wtcols.add(wtcolx)
 
-                if cty == xlrd.XL_CELL_EMPTY:
+                if cty == m3_xlrd.XL_CELL_EMPTY:
                     continue
                 # XF - индексы
                 if cell.xf_index is not None:
@@ -210,7 +225,7 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
                 # если поле текстовое и
                 # стоит настройка "Сохранять текстовые поля"
                 # то не преобразуем текст в число
-                if KEEP_TEXT_TYPE and format_str == TEXT_CELL_FORMAT:
+                if keep_text_type and format_str == TEXT_CELL_FORMAT:
                     pass
                 else:
                     try:
@@ -218,8 +233,11 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
                         if isinstance(val1, float):
                             val1 = str(val1)
                         decimal.Decimal(val1)
-                        cty = xlrd.XL_CELL_NUMBER
-                    except (decimal.InvalidOperation, TypeError):
+                        cty = m3_xlrd.XL_CELL_NUMBER
+                    except (
+                            decimal.InvalidOperation,
+                            TypeError,
+                            UnicodeEncodeError):
                         pass
                 runlist = self.writer.rdsheet.rich_text_runlist_map.get(
                     (rdrowx, rdcolx)
@@ -274,16 +292,16 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
         определить, является ли ячейка числовой
         """
         cty = cell_type
-        if KEEP_TEXT_TYPE and cell_type == xlrd.XL_CELL_TEXT:
+        if KEEP_TEXT_TYPE and cell_type == m3_xlrd.XL_CELL_TEXT:
             return cty
         try:
-            long(value)
-            cty = xlrd.XL_CELL_NUMBER
+            int(value)
+            cty = m3_xlrd.XL_CELL_NUMBER
         except ValueError:
             pass
         return cty
 
-    def get_value_type(self, value, default_type=xlrd.XL_CELL_TEXT):
+    def get_value_type(self, value, default_type=m3_xlrd.XL_CELL_TEXT):
         """
          Возвращаем тип значения для выходного элемента
         :param value: значение
@@ -291,25 +309,25 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
         :result: тип ячейки
         """
 
-        if isinstance(value, basestring):
-            cty = xlrd.XL_CELL_TEXT
+        if isinstance(value, six.string_types):
+            cty = m3_xlrd.XL_CELL_TEXT
         elif isinstance(value, (datetime, date, time)):
-            cty = xlrd.XL_CELL_DATE
+            cty = m3_xlrd.XL_CELL_DATE
         elif isinstance(value, bool):
-            cty = xlrd.XL_CELL_BOOLEAN
+            cty = m3_xlrd.XL_CELL_BOOLEAN
         elif value is None:
-            cty = xlrd.XL_CELL_EMPTY
+            cty = m3_xlrd.XL_CELL_EMPTY
         # elif isinstance(value, numbers.Number):
-        #     if default_type == xlrd.XL_CELL_TEXT and KEEP_TEXT_TYPE:
+        #     if default_type == m3_xlrd.XL_CELL_TEXT and KEEP_TEXT_TYPE:
         #         return default_type
-        #     cty = xlrd.XL_CELL_NUMBER
+        #     cty = m3_xlrd.XL_CELL_NUMBER
         else:
             cty = default_type
-            # if default_type == xlrd.XL_CELL_TEXT and KEEP_TEXT_TYPE:
+            # if default_type == m3_xlrd.XL_CELL_TEXT and KEEP_TEXT_TYPE:
             #     return cty
             # try:
             #     long(value)
-            #     cty = xlrd.XL_CELL_NUMBER
+            #     cty = m3_xlrd.XL_CELL_NUMBER
             # except ValueError:
             #     cty = default_type
 
@@ -358,7 +376,7 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
         if not wt_font:
             wt_font = self.create_font(font_index)
             self.fonts[font_index] = wt_font
-            return wt_font
+        return wt_font
 
     def create_font(self, rd_font_index):
         """
@@ -388,7 +406,7 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
         return wtf
 
     def write_result(
-        self, write_coords, value, style, cell_type, (runlist, rdrowx, rdcolx)
+        self, write_coords, value, style, cell_type, extra
     ):
         """
          Выводим в ячейку с координатами `write_coords`
@@ -402,6 +420,7 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
         :param rdrowx: строка в исходном файле
         :param rdcolx: колонка в исходном файле
         """
+        runlist, rdrowx, rdcolx = extra
         wtcolx, wtrowx = write_coords
         if cell_type == EXCEL_IMAGE_TYPE:
             self.writer.wtsheet.insert_bitmap(
@@ -418,7 +437,7 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
         if cell_type == FORMULA_XLS_TYPE:
             self.writer.wtsheet.write(wtrowx, wtcolx, xlwt.Formula(value),
                                       style)
-        elif cell_type == xlrd.XL_CELL_TEXT or cell_type == xlrd.XL_CELL_EMPTY:
+        elif cell_type == m3_xlrd.XL_CELL_TEXT or cell_type == m3_xlrd.XL_CELL_EMPTY:
 
 
             if runlist is not None:
@@ -430,15 +449,15 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
                     wtrowx, wtcolx, rich_text_list, style=style)
             else:
                 wtrow.set_cell_text(wtcolx, value, style)
-        elif cell_type == xlrd.XL_CELL_NUMBER:
+        elif cell_type == m3_xlrd.XL_CELL_NUMBER:
             wtrow.set_cell_number(wtcolx, value, style)
-        elif cell_type == xlrd.XL_CELL_DATE:
+        elif cell_type == m3_xlrd.XL_CELL_DATE:
             wtrow.set_cell_text(wtcolx, value, style)
-        elif cell_type == xlrd.XL_CELL_BLANK:
+        elif cell_type == m3_xlrd.XL_CELL_BLANK:
             wtrow.set_cell_blank(wtcolx, style)
-        elif cell_type == xlrd.XL_CELL_BOOLEAN:
+        elif cell_type == m3_xlrd.XL_CELL_BOOLEAN:
             wtrow.set_cell_boolean(wtcolx, value, style)
-        elif cell_type == xlrd.XL_CELL_ERROR:
+        elif cell_type == m3_xlrd.XL_CELL_ERROR:
             wtrow.set_cell_error(wtcolx, value, style)
         else:
             raise XLSReportWriteException
