@@ -1,9 +1,13 @@
 # coding: utf-8
-__author__ = 'prefer'
+from __future__ import absolute_import
 
-from simple_report.core.cursor import AbstractCursor, AbstractCalculateNextCursor
+import six
+
+from simple_report.core.cursor import AbstractCalculateNextCursor
+from simple_report.core.cursor import AbstractCursor
 from simple_report.utils import ColumnHelper
 
+
 class Cursor(AbstractCursor):
     """
     Специализированный курсор для XLSX таблиц.
@@ -34,7 +38,7 @@ class Cursor(AbstractCursor):
         # Координаты в XLSX таблицах имеют вид
         # (F, 3). F - имя стобла
         #         3 - номер строки. Нумерация строк с 1
-        assert isinstance(value[0], basestring)
+        assert isinstance(value[0], six.string_types)
 
 
 class CalculateNextCursorXLSX(AbstractCalculateNextCursor):
