@@ -1,8 +1,10 @@
 # coding: utf-8
-from simple_report.core.document_wrap import DocumentOpenXML, SpreadsheetDocument
+from __future__ import absolute_import
+
+from simple_report.core.document_wrap import DocumentOpenXML
+from simple_report.core.document_wrap import SpreadsheetDocument
 from simple_report.xlsx.spreadsheet_ml import CommonPropertiesXLSX
 
-__author__ = 'prefer'
 
 class DocumentXLSX(DocumentOpenXML, SpreadsheetDocument):
     u"""
