@@ -1,24 +1,37 @@
 # coding: utf-8
+from __future__ import absolute_import
 
-import copy
 from datetime import datetime
-from decimal import Decimal
+import copy
+import numbers
 import re
-from lxml.etree import QName, SubElement
+
+from lxml.etree import QName
+from lxml.etree import SubElement
+from six.moves import range
+import six
+
+from simple_report.core.exception import SheetDataException
 from simple_report.core.shared_table import SharedStringsTable
+from simple_report.core.spreadsheet_section import AbstractMerge
+from simple_report.core.spreadsheet_section import SpreadsheetSection
 from simple_report.core.tags import TemplateTags
 from simple_report.interface import ISpreadsheetSection
-from simple_report.utils import (ColumnHelper, get_addr_cell, date_to_float,
-    FormulaWriteExcel)
-from simple_report.xlsx.cursor import Cursor
-from simple_report.core.spreadsheet_section import SpreadsheetSection, AbstractMerge
-from simple_report.core.exception import SheetDataException
-from simple_report.xlsx.formula import Formula
+from simple_report.utils import ColumnHelper
+from simple_report.utils import FormulaWriteExcel
+from simple_report.utils import FormulaWriteExcelFromStr
+from simple_report.utils import date_to_float
+from simple_report.utils import get_addr_cell
+from simple_report.utils import split_string
 from simple_report.xlsx.cursor import CalculateNextCursorXLSX
+from simple_report.xlsx.cursor import Cursor
+from simple_report.xlsx.formula import Formula
 
-__author__ = 'prefer'
 
 XLSX_GROUPING_COUNT = 30
+ORIENTATION_LANDSCAPE = 'landscape'
+ORIENTATION_PORTRAIT = 'portrait'
+MAX_COLUMN_NAME_LEN = 3  # Максимальная колонка - XFD (2^14)
 
 
 class SheetData(object):
@@ -101,6 +114,8 @@ class SheetData(object):
 
         # Ссылка на тег данных строк и столбцов листа с очищенными значениями
         self.write_data = self._write_xml.find(QName(self.ns, 'sheetData'))
+        self._page_setup = self._write_xml.find(QName(self.ns, 'pageSetup'))
+
         self.write_data_dict = {}
         if not self.write_data is None:
             self.write_data.clear()
@@ -172,6 +187,15 @@ class SheetData(object):
         assert isinstance(value, Cursor)
         self._last_section = value
 
+    @property
+    def default_row_height(self):
+        u"""
+        Высота строки по умолчанию
+        """
+        sheet_format = self._read_xml.find(QName(self.ns, 'sheetFormatPr'))
+        if sheet_format is not None:
+            return float(sheet_format.attrib.get('defaultRowHeight', 0))
+
     def flush(self, begin, end, start_cell, params, used_formulas=None):
         """
         Вывод секции
@@ -180,8 +204,20 @@ class SheetData(object):
         :param start_cell: ячейка с которой надо выводить
         :param params: данные для вывода
         """
+
         self.set_section(begin, end, start_cell, params, used_formulas)
         self.set_merge_cells(begin, end, start_cell)
+
+        self.last_section.row = (
+            self.cursor.row[0],
+            max((self.cursor.row[1], self.last_section.row[1]))
+        )
+
+        self.last_section.column = (
+            max((self.cursor.column[0], self.last_section.column[0])),
+            self.cursor.column[1]
+        )
+
         self.set_dimension()
 
         if self.write_cols is not None:
@@ -193,8 +229,8 @@ class SheetData(object):
         u"""
         Установка диапазона
         """
-        _, row_index = self.cursor.row
-        col_index, _ = self.cursor.column
+        _, row_index = self.last_section.row
+        col_index, _ = self.last_section.column
 
         dimension = 'A1:%s' % \
                     (ColumnHelper.add(col_index, -1) + str(row_index - 1))
@@ -415,19 +451,10 @@ class SheetData(object):
             cell_el = SubElement(row_el, 'c', attrib=attrib_cell)
         return cell_el
 
-    # def _add_formula_to_calc_chain(self, row, column):
-    #     """
-    #     Добавление формулы в цепочку вычислений (если файл есть в
-    #         шаблоне)
-    #     """
-    #     if self.calc_chain:
-    #         calc = SubElement(self.calc_chain._root, 'c')
-    #         calc.attrib['r'] = '%s%s' % (column, row)
-    # @profile
     def set_section(self, begin, end, start_cell, params, used_formulas=None):
         u"""
         Вывод секции
-        
+
         :param begin: начальная ячейка секции
         :type begin: 2-tuple
         :param end: конечная ячейка секции
@@ -446,7 +473,7 @@ class SheetData(object):
             used_formulas = {}
 
         for i, num_row, row in self._find_rows(range_rows):
-            attrib_row = dict(row.items())
+            attrib_row = dict(list(row.items()))
 
             row_index = start_row + i
             attrib_row['r'] = str(row_index)
@@ -454,7 +481,7 @@ class SheetData(object):
             row_el = self._create_or_get_output_row(row_index, attrib_row)
 
             for j, col, cell in self._find_cells(range_cols, num_row, row):
-                attrib_cell = dict(cell.items())
+                attrib_cell = dict(list(cell.items()))
 
                 col_index = ColumnHelper.add(start_column, j)
 
@@ -520,7 +547,7 @@ class SheetData(object):
                                         # Дата конвертируется в int, начиная с 31.12.1899
                                         is_int = True
                                         cell_el.attrib['t'] = 'n'  # type - number
-                                        value_el.text = unicode(days)
+                                        value_el.text = six.text_type(days)
                                     else:
                                         date_less_1900 = '%s.%s.%s' % (
                                             param_value.date().day,
@@ -528,24 +555,20 @@ class SheetData(object):
                                             param_value.date().year
                                         )
                                         # strftime(param_value, locale.nl_langinfo(locale.D_FMT)) - неработает для 1900 и ниже
-                                        value_string = value_string.replace(found_param, unicode(date_less_1900))
+                                        value_string = value_string.replace(found_param, six.text_type(date_less_1900))
 
-                                # Добавил long, возможно нужно использовать
-                                # общего предка numbers.Number
-                                elif isinstance(
-                                    param_value,
-                                    (int, float, long, Decimal)
-                                ) and found_param == value_string:
+                                elif (isinstance(param_value, numbers.Number) and
+                                      found_param == value_string):
                                     # В первую очередь добавляем числовые значения
                                     is_int = True
 
                                     cell_el.attrib['t'] = 'n'  # type - number
-                                    value_el.text = unicode(param_value)
+                                    value_el.text = six.text_type(param_value)
 
-                                elif isinstance(param_value, basestring):
+                                elif isinstance(param_value, six.string_types):
                                     # Строковые параметры
 
-                                    value_string = value_string.replace(found_param, unicode(param_value))
+                                    value_string = value_string.replace(found_param, six.text_type(param_value))
 
                                 elif isinstance(param_value, FormulaWriteExcel):
                                     # Записываем формулу, которой не было в
@@ -570,7 +593,7 @@ class SheetData(object):
                                         elif f_cells:
                                             # Группируем по 30 элементов
                                             cells_number = int(
-                                                len(f_cells) / XLSX_GROUPING_COUNT
+                                                len(f_cells) // XLSX_GROUPING_COUNT
                                             ) + 1
                                             val_list = []
                                             for x in range(cells_number):
@@ -594,16 +617,28 @@ class SheetData(object):
                                     else:
                                         continue
 
+                                elif isinstance(
+                                        param_value, FormulaWriteExcelFromStr):
+                                    # Записываем кастомную формулу из строки
+                                    attrib_cell['t'] = 'e'
+                                    # тип вычислимого по формуле поля
+                                    cell_el.remove(value_el)
+                                    # значения в ячейке с формулой
+                                    formula_el = SubElement(cell_el, 'f')
+                                    formula_el.text = param_value.text
+
                                 elif param_value:
                                     # любой объект, например список или
                                     # словарь. Вынесено для производительности
 
-                                    value_string = value_string.replace(found_param, unicode(param_value))
+                                    value_string = value_string.replace(found_param, six.text_type(param_value))
 
                                 else:
                                     # Не передано значение параметра
                                     value_string = value_string.replace(found_param, '')
-
+                            value_string = adjust_row_height_and_split_string(
+                                row_el, value_string, Section.MIN_ROW_HEIGHT
+                            )
                             if not is_int:
                                 # Добавим данные в shared strings
 
@@ -636,7 +671,7 @@ class SheetData(object):
         rows = begin[1], end[1] + 1
         cols = begin[0], end[0]
 
-        range_rows = xrange(*rows)
+        range_rows = range(*rows)
         range_cols = list(ColumnHelper.get_range(*cols))
 
         return range_rows, range_cols
@@ -653,9 +688,12 @@ class SheetData(object):
         :type end: 2-tuple
         """
         col, row = addr
-        rows = xrange(begin[1], end[1] + 1)
-        cols = list(ColumnHelper.get_range(begin[0], end[0]))
-        return all([col in cols, row in rows])
+        rows = range(begin[1], end[1] + 1)
+
+        return (
+            (row in rows) &
+            (col in list(ColumnHelper.get_range(begin[0], end[0])))
+        )
 
     def get_cell_end(self, cell_addr):
         """
@@ -747,13 +785,13 @@ class SheetData(object):
             if end_col - begin_col > 0:
                 if col_index > begin_col:
                     # предыдущий интервал: max = col-1
-                    prev_attrib_col = dict(dst_col.items())
+                    prev_attrib_col = dict(list(dst_col.items()))
                     prev_attrib_col['max'] = str(col_index - 1)
                     prev_col_el = SubElement(self.write_cols, 'col', attrib=prev_attrib_col)
 
                 if col_index < end_col:
                     # следующий интервал: min = col+1
-                    next_attrib_col = dict(dst_col.items())
+                    next_attrib_col = dict(list(dst_col.items()))
                     next_attrib_col["min"] = str(col_index + 1)
                     next_col_el = SubElement(self.write_cols, 'col', attrib=next_attrib_col)
 
@@ -788,7 +826,7 @@ class SheetData(object):
             # если нет исходной колонки, то не надо копировать
             if not src_col_el is None:
                 # копируем данные
-                attrib_col = dict(src_col_el.items())
+                attrib_col = dict(list(src_col_el.items()))
                 dst_col_el = self._create_or_get_output_col(dst_col, attrib_col)
                 # записываем в новую колонку
                 self._set_new_column_width(dst_col, src_col_el, dst_col_el)
@@ -908,9 +946,12 @@ class SheetData(object):
         :param end_new_merge: конец диапазона
         :type end_new_merge: 2-tuple
         """
+        def align_chars(col):
+            """Выравнивает имя колонки для выполнения сравнений"""
+            return col.zfill(MAX_COLUMN_NAME_LEN)
 
-        begin_new_column, begin_new_row = get_addr_cell(begin_new_merge)
-        end_new_column, end_new_row = get_addr_cell(end_new_merge)
+        begin_new_col, begin_new_row = get_addr_cell(begin_new_merge)
+        end_new_col, end_new_row = get_addr_cell(end_new_merge)
 
         # Все смерженные ячейки на листе
         merge_cells = self._write_xml.xpath('.//mergeCell')
@@ -919,20 +960,37 @@ class SheetData(object):
             ref_attr = merge_cell.attrib.get('ref')  # D1:D3 Например
             begin_old_merge, end_old_merge = ref_attr.split(':')  # D1, D3
 
-            begin_old_column, begin_old_row = get_addr_cell(begin_old_merge)
-            end_old_column, end_old_row = get_addr_cell(end_old_merge)
+            begin_old_col, begin_old_row = get_addr_cell(begin_old_merge)
+            end_old_col, end_old_row = get_addr_cell(end_old_merge)
 
-            if not (begin_new_column > end_old_column or end_new_column < begin_old_column or begin_new_row > end_old_row or
-                end_new_row < begin_old_row):
+            if not (
+                    align_chars(begin_new_col) > align_chars(end_old_col) or
+                    align_chars(end_new_col) < align_chars(begin_old_col) or
+                    begin_new_row > end_old_row or
+                    end_new_row < begin_old_row):
 
                 self.write_merge_cell.remove(merge_cell)
 
+    def set_orientation(self, orientation):
+        """
+        Задать ориентацию страницы для данного листа
+        :param orientation: 'portrait' или 'landscape'
+        """
+        if self._page_setup is not None:
+            assert orientation in (
+                ORIENTATION_LANDSCAPE,
+                ORIENTATION_PORTRAIT
+            )
+            self._page_setup.attrib['orientation'] = orientation
 
 class Section(SpreadsheetSection, ISpreadsheetSection):
     u"""
     Секция отчета
     """
 
+    MIN_ROW_HEIGHT = 6.
+    DEFAULT_ROW_HEIGHT = 15.
+
     def __init__(self, sheet_data, name, begin=None, end=None):
         """
         :param sheet_data: Данные листа
@@ -954,6 +1012,17 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
     def __repr__(self):
         return self.__str__()
 
+    def get_copy(self):
+        u"""
+        Получение копии секции, в которой можно манипулировать
+        входными данными без побочных эффектов на другие секции
+        :param self:
+        :return:
+        """
+        section_copy = copy.copy(self)
+        section_copy.sheet_data = copy.copy(self.sheet_data)
+        section_copy.sheet_data.read_data = copy.copy(self.sheet_data.read_data)
+        return section_copy
 
     def flush(
         self, params, oriented=ISpreadsheetSection.LEFT_DOWN,
@@ -962,7 +1031,7 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
         """
         Вывод. Имеется два механизма вывода.
         Для использования старого не передавать direction
-        
+
         :param params: параметры замены
         :type params: dict
         :param oriented: направление ориентации
@@ -978,23 +1047,35 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
                             Section.RIGHT,
                             Section.HIERARCHICAL)
 
-        # Тут смещение курсора, копирование данных из листа и общих строк
-        # Генерация новых данных и новых общих строк
+        if oriented == Section.HIERARCHICAL:
+            # Тут смещение курсора, копирование данных из листа и общих строк
+            # Генерация новых данных и новых общих строк
 
-        cursor = self.sheet_data.cursor
+            cursor = self.sheet_data.cursor
 
-        begin_col, begin_row = self.begin
-        end_col, end_row = self.sheet_data.get_cell_end(self.end)
+            begin_col, begin_row = self.begin
+            end_col, end_row = self.sheet_data.get_cell_end(self.end)
 
-        current_col, current_row = CalculateNextCursorXLSX().get_next_cursor(cursor, (begin_col, begin_row),
-                        (end_col, end_row), oriented, self)
+            current_col, current_row = CalculateNextCursorXLSX().get_next_cursor(
+                cursor, (begin_col, begin_row),
+                (end_col, end_row), oriented, self
+            )
+        else:
+            current_col, current_row = (
+                CalculateNextCursorXLSX().get_next_cursor(
+                    self.sheet_data.cursor,
+                    self.begin,
+                    self.sheet_data.get_cell_end(self.end),
+                    oriented, None
+                )
+            )
 
-        self.sheet_data.last_section.row = (current_col, current_row)
-        self.sheet_data.last_section.column = (ColumnHelper.add(current_col,
-        ColumnHelper.difference(end_col, begin_col)), current_row + end_row - begin_row)
-
-        self.sheet_data.flush(self.begin, self.end, (current_col, current_row),
-            params, used_formulas)
+        self.sheet_data.flush(
+            self.begin, self.end,
+            (current_col, current_row),
+            params,
+            used_formulas
+        )
 
     def get_all_parameters(self):
         u"""
@@ -1012,6 +1093,267 @@ class Section(SpreadsheetSection, ISpreadsheetSection):
 
         return ColumnHelper.difference(end_col, begin_col) + 1
 
+    def get_total_height(self):
+        u"""
+        Возвращает полную высоту секции. Если нужна высота секции с учётом текста,
+        то её следует перед вызовом этого метода заполнить при помощи метода fill
+        :return: Высота секции в точках Excel
+        :rtype: int
+        """
+        sheet_data = self.sheet_data
+        rows_range, col_range = sheet_data._range(
+            self.begin, self.end
+        )
+        default_row_height = (
+            sheet_data.default_row_height or
+            self.DEFAULT_ROW_HEIGHT
+        )
+        total_height = 0.
+        for _, _, row in sheet_data._find_rows(rows_range):
+            row_height = row.attrib.get('ht')
+            total_height += (
+                float(row_height) if row_height else default_row_height
+            )
+        return total_height
+
+    def _substitute_param(self, found_param, params, value_string):
+        u"""
+        Подстановка параметров в строку
+        :param found_param: найденный в строке параметр
+        :param params: Словарь параметров
+        :param value_string: Исходная строка
+        :return: Строка с подставленными параметрами
+        :rtype: six.text_type
+        """
+        param_name = found_param[1:-1]
+        param_value = params.get(param_name)
+        sheet_data = self.sheet_data
+
+        value_type = 's'
+
+        if (param_name[0] == sheet_data.PREFIX_TAG and
+                param_name[-1] == sheet_data.PREFIX_TAG):
+            param_value = sheet_data.tags.get(param_name[1:-1])
+
+        if (isinstance(param_value, datetime) and
+                found_param == value_string):
+            # В OpenXML хранится дата относительно 1900 года
+            days = date_to_float(param_value)
+            if days > 0:
+                # Дата конвертируется в int, начиная с
+                # 31.12.1899
+                value_type = 'n'
+                value_string = six.text_type(days)
+            else:
+                date_less_1900 = '%s.%s.%s' % (
+                    param_value.date().day,
+                    param_value.date().month,
+                    param_value.date().year
+                )
+                value_string = value_string.replace(
+                    found_param, six.text_type(date_less_1900))
+
+        elif (isinstance(param_value, numbers.Number) and
+                found_param == value_string):
+            value_string = six.text_type(param_value)
+            value_type = 'n'
+            # В первую очередь добавляем числовые значения
+
+        elif isinstance(param_value, six.string_types):
+            # Строковые параметры
+
+            value_string = value_string.replace(
+                found_param, six.text_type(param_value))
+
+        elif param_value:
+            # любой объект, например список или
+            # словарь. Вынесено для производительности
+
+            value_string = value_string.replace(
+                found_param, six.text_type(param_value))
+
+        else:
+            # Не передано значение параметра
+            value_string = value_string.replace(
+                found_param, '')
+
+        return value_string, value_type
+
+    def _write_to_cell(self, cell, value_string, value_type):
+        u"""
+        Запись значения в ячейку
+        :param cell: XML объект ячейки
+        :param value_string: Содержимое ячейки в виде строки
+        :param value_type: тип содержимого ячейки:
+                           's' (строка) или 'n' (число)
+        :return: None
+        """
+        sheet_data = self.sheet_data
+        is_int = (value_type == 'n')
+        if is_int:
+            cell.attrib['t'] = value_type
+
+            value_element = cell.find(
+                'v',
+                namespaces={None: self.sheet_data.ns}
+            )
+            if value_element is not None:
+                value_element.text = value_string
+            else:
+                cell.text = six.text_type(value_string)
+
+        else:
+            # Добавим данные в shared strings
+            sheet_data.shared_table.elements.append(value_string)
+            new_index = len(sheet_data.shared_table.elements) - 1
+            sheet_data._get_tag_value(cell).text = six.text_type(new_index)
+
+    def fill(self, params):
+        u"""
+        Подстановка параметров in-place.
+        Пример использования: перед выводом секции нужно проверить, что
+        она помещается на страницу
+
+        section_copy = section.get_copy()
+        section_copy.fill(params)
+        section_height = section_copy.get_total_height()
+        if( ... ):
+            section_copy.flush({})  # все данные уже зашиты в section_copy
+        """
+        sheet_data = self.sheet_data
+        range_rows, range_cols = sheet_data._range(self.begin, self.end)
+
+        for i, num_row, row in sheet_data._find_rows(range_rows):
+            for j, col, cell in sheet_data._find_cells(range_cols, num_row, row):
+                attrib_cell = dict(cell.items())
+
+                col_index = col
+
+                attrib_cell['r'] = col_index + str(num_row)
+
+                value = sheet_data._get_tag_value(cell)
+                if value is None:
+                    continue
+
+                if attrib_cell.get('t') in ('n', None):  # number
+                    pass
+
+                elif attrib_cell.get('t') == 's':
+                    # 't' = 's' - значит есть значения shared strings
+                    index_value = int(value.text)
+                    value_string = sheet_data.shared_table.get_value(index_value)
+
+                    who_found_params = sheet_data._get_values_by_re(value_string) or []
+
+                    for found_param in who_found_params:
+
+                        value_string, value_type = self._substitute_param(
+                            found_param,
+                            params,
+                            value_string
+                        )
+
+                        value_string = adjust_row_height_and_split_string(
+                            row, value_string, self.MIN_ROW_HEIGHT
+                        )
+
+                        self._write_to_cell(cell, value_string, value_type)
+
+                elif attrib_cell.get('t'):
+                    raise SheetDataException(
+                        "Unknown value '%s' for tag t" % attrib_cell.get('t'))
+
+
+def adjust_row_height_and_split_string(row_el, value_string, min_row_height):
+    u"""
+    Реализация возможности подбора высоты строки.
+
+    В строке должен находится маркер вида:
+
+      @@max_length:line_height[,offset][:force_split[:sep]]@@
+
+    max_length - Максимальное количество символов
+      в ячейке, после которого нужен перенос.
+
+    line_height - Высота ряда, может быть не целым числом
+       (разделитель - точка)
+
+    offset - фиксированный отступ по вертикали, т.е.
+      полная высота ячейки
+        number_of_lines * line_height + offset
+      по умолчанию offset = 0
+
+    force_split - Флаг (1 или 0). Следует ли разбить
+      строку принудительно.
+
+    sep - Опционально: символ-разделитель.
+      По умолчанию - пробел.
+
+    Пример маркера: @@65:15,3.5:0:.@@
+
+    :param row_el: XML элемент строки
+    :param value_string: Текст в строке
+    :param min_row_height: Минимальная высота строки в точках Excel
+    :return: текст, разбитый на строки
+    :rtype: basestring
+    """
+
+    RE_AUTOHEIGHT_MARKER = re.compile(
+        '@@(?P<max_length>[\d{1,}]+)'
+        ':('
+        '(?P<line_height>[^:,]+)'
+        '(,(?P<offset>[^:]+))?)'
+        '(:(?P<force_split>[^:@]+)'
+        '(:(?P<sep>[^@]+))?'
+        ')?@@'
+    )
+
+    split_settings = RE_AUTOHEIGHT_MARKER.search(
+        value_string)
+
+    if not split_settings:
+        return value_string
+
+    line_max_len = int(
+        split_settings.group('max_length').strip(u'@'))
+    line_height = float(split_settings.group('line_height'))
+
+    offset = float(split_settings.group('offset') or 0)
+
+    force_split = int(split_settings.group('force_split') or 0)
+
+    # TODO: Параметр len_sep больше не используется. Исчез в коммите ab7f63cd
+    #  от Tue Aug 7 16:15:38 2018 по непонятной причине. Возвращать
+    #  использование обратно опасно, так как могут непредсказуемо поехать
+    #  существующие печатки - нужно всё проверить.
+    # len_sep = split_settings.group('sep')
+
+    value_string = value_string.replace(
+        split_settings.group(0), '')
+
+    height = offset
+    result_lines = []
+
+    # Идём по уже разбитым знаком '\n' строкам и суммируем их высоту
+    for oneline in value_string.split('\n'):
+        splited_sublines = split_string(oneline, line_max_len)
+        sublines_count = len(splited_sublines)
+
+        if force_split:
+            # Принудительно разбиваем строку на несколько подстрок
+            result_lines.append('\n'.join(splited_sublines))
+        else:
+            result_lines.append(oneline)
+
+        height += sublines_count * line_height
+
+    current_height = float(
+        row_el.attrib.get('ht', min_row_height))
+    row_el.attrib['ht'] = str(max(height, current_height))
+    row_el.attrib['customHeight'] = '1'
+
+    return '\n'.join(result_lines)
+
 
 class MergeXLSX(AbstractMerge):
     u"""
