@@ -1,21 +1,25 @@
 # coding: utf-8
-'''
-Created on 24.11.2011
+from __future__ import absolute_import
 
-@author: prefer
-'''
-
-import re
 import os
+import re
 
-from lxml.etree import QName, tostring, fromstring
-from simple_report.core import XML_DEFINITION
-from simple_report.core.xml_wrap import OpenXMLFile, ReletionOpenXMLFile, ReletionTypes, CommonProperties
+from lxml.etree import QName
+from lxml.etree import fromstring
+from lxml.etree import tostring
+
+from simple_report.core.exception import SectionException
+from simple_report.core.exception import SectionNotFoundException
+from simple_report.core.exception import SheetNotFoundException
 from simple_report.core.shared_table import SharedStringsTable
-from simple_report.core.exception import SectionNotFoundException, SectionException, SheetNotFoundException
+from simple_report.core.xml_wrap import CommonProperties
+from simple_report.core.xml_wrap import OpenXMLFile
+from simple_report.core.xml_wrap import ReletionOpenXMLFile
+from simple_report.core.xml_wrap import ReletionTypes
 from simple_report.utils import get_addr_cell
 from simple_report.xlsx.cursor import Cursor
-from simple_report.xlsx.section import Section, SheetData
+from simple_report.xlsx.section import Section
+from simple_report.xlsx.section import SheetData
 
 
 class Comments(OpenXMLFile):
@@ -75,8 +79,8 @@ class Comments(OpenXMLFile):
         u"""
         Создание объектов секций из комментариев
         """
-
-        map(self._add_section, self._parse_sections(self.comment_list))
+        for section in  self._parse_sections(self.comment_list):
+            self._add_section(section)
 
     def _add_section(self, values):
         text = values[0]
@@ -151,8 +155,10 @@ class Comments(OpenXMLFile):
         if len(self.comment_list) > 0:
             self.comment_list.clear()
 
-        with open(self.file_path, 'w') as f:
-            f.write(XML_DEFINITION + tostring(self._root))
+        with open(self.file_path, 'wb') as f:
+            f.write(
+                tostring(self._root, encoding='utf-8', xml_declaration=True)
+            )
 
 
 class SharedStrings(OpenXMLFile):
@@ -171,8 +177,10 @@ class SharedStrings(OpenXMLFile):
         Сборка файла
         """
         new_root = self.table.to_xml()
-        with open(self.file_path, 'w') as f:
-            f.write(XML_DEFINITION + tostring(new_root))
+        with open(self.file_path, 'wb') as f:
+            f.write(
+                tostring(new_root, encoding='utf-8', xml_declaration=True)
+            )
 
 
 class WorkbookSheet(ReletionOpenXMLFile):
@@ -273,8 +281,10 @@ class WorkbookSheet(ReletionOpenXMLFile):
         """
         new_root = self.sheet_data.new_sheet()
 
-        with open(self.file_path, 'w') as f:
-            f.write(XML_DEFINITION + tostring(new_root))
+        with open(self.file_path, 'wb') as f:
+            f.write(
+                tostring(new_root, encoding='utf-8', xml_declaration=True)
+            )
 
         if self.comments:
             self.comments.build()
@@ -421,7 +431,8 @@ class Workbook(ReletionOpenXMLFile):
     def build(self):
         """
         """
-        map(lambda x: x.build(), self.sheets)
+        for sheet in self.sheets:
+            sheet.build()
 
         self.shared_strings.build()
         if self.calc_chain:
