@@ -1,39 +1,46 @@
-#coding: utf-8
+# coding: utf-8
 
 from datetime import datetime
-import sys
 import os
 import subprocess
-sys.path.append('/home/vahotin/dev/simple_report/src')
-sys.path.append(os.path.join(os.path.dirname(__file__), '../'))
+import sys
+import unittest
+
+from oborot import OperationsJournalReportFactory
+from test_oo_wrapper import TestOO
+from test_pagebreaks import TestPagebreaks
+from test_pko import TestPKO
+from test_utils import skip_python26
+
 from simple_report.converter.abstract import FileConverter
 from simple_report.core.tags import TemplateTags
 from simple_report.interface import ISpreadsheetSection
-from simple_report.xlsx.section import Section, MergeXLSX
-from simple_report.xlsx.spreadsheet_ml import (SectionException,
-                                               SectionNotFoundException)
-from simple_report.xlsx.formula import Formula
-from simple_report.utils import ColumnHelper, date_to_float, FormulaWriteExcel
+from simple_report.report import DocumentReport
+from simple_report.report import ReportGeneratorException
+from simple_report.report import SpreadsheetReport
+from simple_report.utils import ColumnHelper
+from simple_report.utils import FormulaWriteExcel
+from simple_report.utils import date_to_float
+from simple_report.xls.document import DocumentXLS
+from simple_report.xls.section import MergeXLS
 from simple_report.xls.section import XLSImage
+from simple_report.xlsx.formula import Formula
+from simple_report.xlsx.section import MergeXLSX
+from simple_report.xlsx.section import Section
+from simple_report.xlsx.spreadsheet_ml import SectionException
+from simple_report.xlsx.spreadsheet_ml import SectionNotFoundException
+
+
+sys.path.append('/home/vahotin/dev/simple_report/src')
+sys.path.append(os.path.join(os.path.dirname(__file__), '../'))
 
 
 sys.path.append('.')
 
-from test_oo_wrapper import TestOO
-from test_utils import skip_python26
-from test_pko import TestPKO
-from oborot import OperationsJournalReportFactory
-from test_pagebreaks import TestPagebreaks
 
 sys.path.append('../')
 
-import os
-import unittest
 
-from simple_report.report import (SpreadsheetReport, ReportGeneratorException,
-                                  DocumentReport)
-from simple_report.xls.document import DocumentXLS
-from simple_report.xls.section import MergeXLS
 
 
 LOREM_IPSUM = (
@@ -1080,4 +1087,3 @@ def insert_formulas(row_insert_formula_section, check_insert_formula_section):
 
 if __name__ == '__main__':
     unittest.main()
-
